/*
Ci.nsIContentPolicy.TYPE_SCRIPT
Ci.nsIContentPolicy.TYPE_IMAGE
Ci.nsIContentPolicy.TYPE_STYLESHEET
Ci.nsIContentPolicy.TYPE_OBJECT
Ci.nsIContentPolicy.TYPE_DOCUMENT
Ci.nsIContentPolicy.TYPE_SUBDOCUMENT
Ci.nsIContentPolicy.TYPE_REFRESH
Ci.nsIContentPolicy.TYPE_XBL
Ci.nsIContentPolicy.TYPE_XMLHTTPREQUEST
Ci.nsIContentPolicy.TYPE_PING

ACCEPT : 1,
REJECT_OTHER : -4,
REJECT_REQUEST : -1,
REJECT_SERVER : -3,
REJECT_TYPE : -2,
TYPE_DATAREQUEST : 11,
TYPE_DOCUMENT : 6,
TYPE_DTD : 13,
TYPE_FONT : 14,
TYPE_IMAGE : 3,
TYPE_MEDIA : 15,
TYPE_OBJECT_SUBREQUEST : 12,
TYPE_OBJECT : 5,
TYPE_OTHER : 1,
TYPE_PING : 10,
TYPE_REFRESH : 8,
TYPE_SCRIPT : 2,
TYPE_STYLESHEET : 4,
TYPE_SUBDOCUMENT : 7,
TYPE_WEBSOCKET : 16,
TYPE_XBL : 9,
TYPE_XMLHTTPREQUEST : 11,
*/

var BlockContent = {};

BlockContent.showOptions = function(e) {
    //window.openDialog('chrome://blockcont/content/options.xul','','chrome,toolbar,modal,resizable=no');
    window.open('chrome://blockcont/content/options.xul','','chrome,toolbar,modal,resizable=no');
};


//the observer will update the properties of the `blockcont` object for better performance
BlockContent.Observer = new function(){
    const prefs = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefBranch);
    const pref = "extensions.mdsy.block.";
    var self = this;
    self.register = function(){
          var observingPrefsActive = false;
          self._branch = prefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
          self._branch.addObserver(pref, self, false);
          //updateButton();
    };
    self.unregister = function(){
          if(!self._branch) return;
          self._branch.removeObserver("", self);
    };
    self.observe = function(aSubject, aTopic, aData){
        //aData is in the form `extensions.mdsy.block.image`
        var key = aData.split(".").pop(); //grab last token
        BlockContent[key] = prefs.getBoolPref(aData);
    };
};


BlockContent.onLoad = function(event) {

    function log(s){
        var consoleService = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
        s = String(s);
        consoleService.logStringMessage(s);
    }

    function getBoolPref(key, defaultValue){
        var value;
        defaultValue = defaultValue || false;
        try{
            value = prefService.getBoolPref(key);
        }catch(ex){
            value = defaultValue;
            //Components.utils.reportError(("getBoolPref() caused error:\n\n" + String(ex)));
        }
        return value;
    }

    function getCharPref(key, defaultValue){
        var value;
        defaultValue = defaultValue || false;
        try{
            value = prefService.getCharPref(key);
        }catch(ex){
            value = defaultValue;
            //Components.utils.reportError(("getCharPref() caused error:\n\n" + String(ex)));
        }
        return value;
    }

    window.removeEventListener("load", BlockContent.onLoad, false);

    Cu.import("resource://gre/modules/XPCOMUtils.jsm");
    Cu.import("resource://gre/modules/Services.jsm");
    
    const prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);

    BlockContent.font   = getBoolPref("extensions.mdsy.block.font");   
    BlockContent.image  = getBoolPref('extensions.mdsy.block.image');  
    BlockContent.media  = getBoolPref("extensions.mdsy.block.media");  
    BlockContent.object = getBoolPref("extensions.mdsy.block.object"); 
    BlockContent.script = getBoolPref('extensions.mdsy.block.script'); 
    BlockContent.style  = getBoolPref('extensions.mdsy.block.style');  
    BlockContent.subdoc = getBoolPref('extensions.mdsy.block.subdoc'); 
    BlockContent.xhr    = getBoolPref("extensions.mdsy.block.xhr");    

    //if changed, takes effect after restart (since we only read its value here)
    const DEBUG = getBoolPref("extensions.mdsy.block.debug");

    //buffer constants to avoid object resolving
    const TYPE_FONT           = Ci.nsIContentPolicy.TYPE_FONT;
    const TYPE_IMAGE          = Ci.nsIContentPolicy.TYPE_IMAGE;
    const TYPE_MEDIA          = Ci.nsIContentPolicy.TYPE_MEDIA;
    const TYPE_OBJECT         = Ci.nsIContentPolicy.TYPE_OBJECT
    const TYPE_SCRIPT         = Ci.nsIContentPolicy.TYPE_SCRIPT;
    const TYPE_STYLESHEET     = Ci.nsIContentPolicy.TYPE_STYLESHEET;
    const TYPE_SUBDOCUMENT    = Ci.nsIContentPolicy.TYPE_SUBDOCUMENT;
    const TYPE_XMLHTTPREQUEST = Ci.nsIContentPolicy.TYPE_XMLHTTPREQUEST;
    const ACCEPT              = Ci.nsIContentPolicy.ACCEPT;
    const REJECT              = Ci.nsIContentPolicy.REJECT_TYPE; //or maybe Ci.nsIContentPolicy.REJECT_REQUEST
                                                                          
    var ourClass = Cc["@blocker.mdsy/cont-policy;1"];
    
    if(ourClass){
        return; //already registered (safeguard when opening new windows)
    }

    let policy = {

    	// nsIContentPolicy interface implementation

    	classDescription: "content blocker by mdsy",
    	classID: Components.ID("{783880d5-6c75-d872-1f49-378bf5e6e8ff}"),
    	contractID: "@blocker.mdsy/cont-policy;1",
    	xpcom_categories: ["content-policy"],

    	shouldLoad: function(contentType, contentLocation, requestOrigin, node, mimeTypeGuess, extra){
            /*log([contentType,contentLocation.spec, requestOrigin.spec, context, mimeTypeGuess, extra].join('\n'));
    		log("shouldLoad: " + contentType + " " + (contentLocation ? contentLocation.spec : "null") + " " +
    			 (requestOrigin ? requestOrigin.spec : "null") + " " + node + " " + mimeTypeGuess + "\n");*/
            try{

        		let blackList   = getCharPref("extensions.mdsy.block.blacklist");

                let result = ACCEPT;

                let blocks = [BlockContent.font,
                              BlockContent.image,
                              BlockContent.media,
                              BlockContent.object,
                              BlockContent.script,
                              BlockContent.style,
                              BlockContent.subdoc,
                              BlockContent.xhr];

                let types  = [TYPE_FONT,
                              TYPE_IMAGE,
                              TYPE_MEDIA,
                              TYPE_OBJECT,
                              TYPE_SCRIPT,
                              TYPE_STYLESHEET,
                              TYPE_SUBDOCUMENT,
                              TYPE_XMLHTTPREQUEST];

                let i, len = blocks.length;

                if(contentLocation && contentLocation.spec && /^http/.test(contentLocation.spec) ){
                    for(i=0; i<len; i++){
                        if(blocks[i] && (contentType == types[i])){
                            result = REJECT;
                        }
                    }
                    if(blackList){
                        let rx = new RegExp(blackList, 'i');
                        if(rx.test(contentLocation.spec)){
                            result = REJECT;        //this will block EVERYTHING, should be replaced with a loop like above...
                            if(DEBUG) log('BLOCKED URL: ' + contentLocation.spec);                        
                        }
                    }
                }

                if(DEBUG && result==REJECT) log('BLOCKED: ' + contentLocation.spec);

                return result;

            }catch(ex){
                if(DEBUG) log('[Blocker] Error while examining: ' + contentLocation.spec);
                return ACCEPT;
            }

    	},

    	shouldProcess: function(contentType, contentLocation, requestOrigin, node, mimeTypeGuess, extra){
    		return Ci.nsIContentPolicy.ACCEPT;
    	},

        /*
        onChannelRedirect: function ContentPolicy_onChannelRedirect(oldChannel, newChannel, flags){
        },

        asyncOnChannelRedirect: function(oldChannel, newChannel, flags, callback){
            this.onChannelRedirect(oldChannel, newChannel, flags);
            callback.onRedirectVerifyCallback(Components.results.NS_OK);
        },*/

    	// nsIFactory interface implementation
    	createInstance: function(outer, iid){
    		if (outer)
    			throw Cr.NS_ERROR_NO_AGGREGATION;
    		return this.QueryInterface(iid);
    	},

    	// nsISupports interface implementation
    	QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentPolicy, Ci.nsIFactory]),
    	//QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentPolicy, Ci.nsIObserver, Ci.nsIChannelEventSink, Ci.nsIFactory]),
    	
    	
    	//not part of the implementation, but an initializer
    	init: function(){
    		let registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
    		registrar.registerFactory(this.classID, this.classDescription, this.contractID, this);

    		let catMan = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
    		for each (let category in this.xpcom_categories){
    			catMan.addCategoryEntry(category, this.contractID, this.contractID, false, true);
    		}

    	},

    	terminate: function(){
    	    let catMan = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
			for each (let category in this.xpcom_categories){
				catMan.deleteCategoryEntry(category, this.contractID, false);
			}
			let registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
			//does NOT work like this: registrar.unregisterFactory(this.classID, this);
    		//MUST be run asynchronously using nsIThreadManager, (bug 753687)
			Services.tm.currentThread.dispatch(
			    function(){
				    registrar.unregisterFactory(this.classID, this);
			    }.bind(this) /* bind function to the `policy` object */
			, Ci.nsIEventTarget.DISPATCH_NORMAL); //=== Services.tm.currentThread.DISPATCH_NORMAL === 0
		
    	},

    	pause: function(){
    	    let catMan = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
			for each (let category in this.xpcom_categories){
				catMan.deleteCategoryEntry(category, this.contractID, false);
			}
		},

    	resume: function(){
    		let catMan = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
    		for each (let category in this.xpcom_categories){
    			catMan.addCategoryEntry(category, this.contractID, this.contractID, false, true);
    		}
		}

    };

    //start blocking right now...
    policy.init();

    //to be used internally (or not used at all)
    BlockContent.start = function(){
        policy.init();
    };

    BlockContent.stop = function(){
        policy.terminate();
    };


    //to be used (in a button for eaxmple) to enable/disable
    BlockContent.pause = function(){
        policy.pause();
        
    };

    BlockContent.resume = function(){
        policy.resume();
    };


};

window.addEventListener("load", BlockContent.onLoad, false);

window.addEventListener('load',   BlockContent.Observer.register,  false);
window.addEventListener('unload', BlockContent.Observer.unregister, false);

